//  
//  Copyright (c) 1999 by Mike Romberg (romberg@fsl.noaa.gov)
//
//  This file may be distributed under terms of the GPL
//
// $Id: diskmeter.cc,v 1.14 2002/04/08 04:36:47 zedpobre Exp $
//

#include "diskmeter.h"
#include "xosview.h"
#include <fstream>
#include <vector>
#include <stdlib.h>

static const char STATFILENAME[] = "/proc/diskstats";
#define MAX_PROCSTAT_LENGTH 2048

DiskMeter::DiskMeter( XOSView *parent, float max ) : FieldMeterGraph(
  parent, 3, "DISK", "READ/WRITE/IDLE")
    {
    read_prev_ = 0;
    write_prev_ = 0;
    maxspeed_ = max;
    getdiskinfo();
    }

DiskMeter::~DiskMeter( void )
    {
    }

void DiskMeter::checkResources( void )
    {
    FieldMeterGraph::checkResources();

    setfieldcolor( 0, parent_->getResource("diskReadColor") );
    setfieldcolor( 1, parent_->getResource("diskWriteColor") );
    setfieldcolor( 2, parent_->getResource("diskIdleColor") );
    priority_ = atoi (parent_->getResource( "diskPriority" ) );
    dodecay_ = parent_->isResourceTrue("diskDecay" );
    useGraph_ = parent_->isResourceTrue( "diskGraph" );
    SetUsedFormat(parent_->getResource("diskUsedFormat"));
    }

void DiskMeter::checkevent( void )
    {
    getdiskinfo();
    drawfields();
    }

void DiskMeter::getdiskinfo( void )
{

  IntervalTimerStop();
  total_ = maxspeed_;
  char buf[MAX_PROCSTAT_LENGTH];
  ifstream stats( STATFILENAME );
  
  if ( !stats ) {
    cerr <<"Can not open file : " <<STATFILENAME <<endl;
    exit( 1 );
  }

  vector<int> fields;
  string line, val;

  unsigned long read  = 0;
  unsigned long write = 0;

  while (!stats.eof() && !stats.fail()) {
    stats.getline(buf, MAX_PROCSTAT_LENGTH-1);
    if (stats.fail()) break;
    line = buf;
    fields.clear();

    size_t pos = 0, start, end;
    
    while (pos < line.length()) {

      start = line.find_first_not_of(" \t", pos);
      if (start < pos || start >= line.length()) {
	pos = line.length();
	break;
      }

      end = line.find_first_of(" \t", start);
      if (end < start || end >= line.length()) 
	end = line.length();
      val = line.substr(start, end-start);

      fields.push_back(atoi(val.data()));

      pos = end;
    }

    if (fields.size() == 14) {
      read  += fields[5];
      write += fields[9];
    }
  }

  float itim = IntervalTimeInMicrosecs();

  // avoid strange values at first call
  if(read_prev_ == 0) read_prev_   = read;
  if(write_prev_ == 0) write_prev_ = write;
    
  // calculate rate in bytes per second
  fields_[0] = ((read  - read_prev_)  * 1e6 * 512) / itim;
  fields_[1] = ((write - write_prev_) * 1e6 * 512) / itim;

  // fix overflow (conversion bug?)
  if (fields_[0] < 0.0)
    fields_[0] = 0.0;
  if (fields_[1] < 0.0)
    fields_[1] = 0.0;
    
  if (fields_[0] + fields_[1] > total_)
    total_ = fields_[0] + fields_[1];

  fields_[2] = total_ - (fields_[0] + fields_[1]);
  
  read_prev_ = read;
  write_prev_ = write;
    
  setUsed((fields_[0]+fields_[1]), total_);
  IntervalTimerStart();
}
